%{
/**
*	WARNING: Do Not Remove This Section
*
*	$RCSfile: parser_etmcc2.y,v $
*	$Revision: 1.17 $
*	$Date: 2006/12/26 11:28:00 $
*	$Author: zapreevis $
*	
*	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
*	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
*	automated verification of properties concerning long-run and instantaneous rewards
*	as well as cumulative rewards.
*	
*	Copyright (C) The University of Twente, 2004-2006.
*	Authors: Maneesh Khattri, Ivan Zapreev
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Our contacts:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: This is a parser for the logic CSL/PCTL/PRCTL/CSRL in MCC.
*/

#include "parser_to_core.h"

#define YYERROR_VERBOSE 1

extern int yylex(void);
%}
%union
{
	int ival;
	char *sval;
	float fval;
	double dval;
	int nval;
	double *pdval;
	bitset *b;
};

%nonassoc <sval> ATOMIC_PROPOSITION
%nonassoc <dval> DOUBLE_VALUE
%nonassoc LEFT_PARENTHESIS RIGHT_PARENTHESIS LEFT_CURLY_BRACKET RIGHT_CURLY_BRACKET LEFT_SQUARE_BRACKET RIGHT_SQUARE_BRACKET GREATER GREATER_OR_EQUAL LESS LESS_OR_EQUAL PRINT ERROR_BOUND OVERFLOW_VAL UNDERFLOW_VAL METHOD_PATH METHOD_STEADY COMMA COMPLEMENT QUIT SET STEADY_STATE_F GAUSS_JACOBI_M GAUSS_SEIDEL_M MAX_ITERATIONS METHOD_UNTIL_REWARDS UNIFORMIZATION_SERICOLA UNIFORMIZATION_QURESHI_SANDERS DISCRETIZATION_TIJMS_VELDMAN SSD ON OFF RESULT STATE
%nonassoc PROBABILITY_F NEXT_F UNTIL_F SPC NEWLINE TTRUE FFALSE IMPLIES EXPECTED_REWARD_RATE_F INSTANTANEOUS_REWARD_F EXPECTED_ACCUMULATED_REWARD_F LONG_RUN_F HELP PROB_THRESHOLD_QURESHI_SANDERS DISCRETIZATION_FACTOR

%left AND OR
%right NOT

%type <ival> mformula
%type <b> stateformula notstateformula bracedstateformula propositionstateformula steadystateformula transientstateformula lstateformula estateformula cstateformula ystateformula
%type <b> notformula formula bracedformula propositionformula steadyformula transientformula eformula cformula yformula lformula
%type <pdval> pathformula untilformula nextformula
/*The following is for {||, &&, >, <, =>, <=}*/
%type <ival> connective comparator

%%
mformula		: NEWLINE
			{
				return 1;
			}
			//WARNING: This is the top most for the FORMULA TREE!
			| stateformula NEWLINE
			{
				//Print the results if required
				setAndPrintResults( $1 );
				return 1;
			}
			| QUIT NEWLINE
			{
				cleanResults();
				return 0;
			}
			| PRINT NEWLINE
			{
				print_runtime_info();
				return 1;
			}
			| SET PRINT ON NEWLINE
			{
				setPrintingStatus(TRUE);
				return 1;
			}
			| SET PRINT OFF NEWLINE
			{
				setPrintingStatus(FALSE);
				return 1;
			}
			| SET ERROR_BOUND DOUBLE_VALUE NEWLINE
			{
				set_error_bound($3);
				return 1;
			}
			| SET MAX_ITERATIONS DOUBLE_VALUE NEWLINE
			{
				set_max_iterations( (int) $3);
				return 1;
			}
			| SET OVERFLOW_VAL DOUBLE_VALUE NEWLINE
			{
				set_overflow($3);
				return 1;
			}
			| SET PROB_THRESHOLD_QURESHI_SANDERS DOUBLE_VALUE NEWLINE
			{
				set_w($3);
				return 1;
			}
			| SET DISCRETIZATION_FACTOR DOUBLE_VALUE NEWLINE
			{
				set_d_factor($3);
				return 1;
			}
			| SET UNDERFLOW_VAL DOUBLE_VALUE NEWLINE
			{
				set_underflow($3);
				return 1;
			}
			| SET METHOD_PATH GAUSS_JACOBI_M NEWLINE
			{
				set_method_path(GJ);
				return 1;
			}
			| SET METHOD_PATH GAUSS_SEIDEL_M NEWLINE
			{
				set_method_path(GS);
				return 1;
			}
			| SET METHOD_STEADY GAUSS_JACOBI_M NEWLINE
			{
				set_method_steady(GJ);
				return 1;
			}
			| SET METHOD_STEADY GAUSS_SEIDEL_M NEWLINE
			{
				set_method_steady(GS);
				return 1;
			}
			| SET METHOD_UNTIL_REWARDS UNIFORMIZATION_SERICOLA NEWLINE
			{
				set_method_until_rewards(US);
				return 1;
			}
			| SET METHOD_UNTIL_REWARDS UNIFORMIZATION_QURESHI_SANDERS NEWLINE
			{
				set_method_until_rewards(UQS);
				return 1;
			}
			| SET METHOD_UNTIL_REWARDS DISCRETIZATION_TIJMS_VELDMAN NEWLINE
			{
				set_method_until_rewards(DTV);
				return 1;
			}
			| SET SSD ON NEWLINE
			{
				set_ssd_on();
				return 1;
			}
			| SET SSD OFF NEWLINE
			{
				set_ssd_off();
				return 1;
			}
			| HELP NEWLINE
			{
				printHelpMessage();
				return 1;
			}
			| RESULT LEFT_SQUARE_BRACKET DOUBLE_VALUE RIGHT_SQUARE_BRACKET NEWLINE
			{
				printResultingStateProbability( (int) $3);
				return 1;
			}
			| STATE LEFT_SQUARE_BRACKET DOUBLE_VALUE RIGHT_SQUARE_BRACKET NEWLINE
			{
				printResultingStateSatisfyability( (int) $3 );
				return 1;
			}
			;
stateformula		: notstateformula		{$$=$1;}
	    		| bracedstateformula		{$$=$1;}
	    		| propositionstateformula	{$$=$1;}
	    		| steadystateformula		{$$=$1;}
	    		| transientstateformula		{$$=$1;}
			| lstateformula			{$$=$1;}
			| estateformula			{$$=$1;}
			| cstateformula			{$$=$1;}
			| ystateformula			{$$=$1;}
	    		;
notstateformula		: notformula			{$$=$1;}
	       		| notformula connective stateformula 
			{
				$$ = doConnective($1, $2, $3);
			}
	       		;
bracedstateformula	: bracedformula			{$$=$1;}
			| bracedformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
propositionstateformula	: propositionformula 		{$$=$1;}
			| propositionformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
steadystateformula	: steadyformula			{$$=$1;}
			| steadyformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
transientstateformula	: transientformula		{$$=$1;}
			| transientformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
lstateformula		: lformula			{$$=$1;}
			| lformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
estateformula		: eformula		{$$=$1;}
			| eformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
cstateformula		: cformula		{$$=$1;}
			| cformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
ystateformula		: yformula		{$$=$1;}
			| yformula connective stateformula
			{
				$$ = doConnective($1, $2, $3);
			}
			;
notformula		: COMPLEMENT formula
			{
				$$ = getComplementBitset( $2 );
			}
	  		;
formula			: notformula		{$$=$1;}
			| bracedformula 	{$$=$1;}
			| propositionformula	{$$=$1;}
			| steadyformula		{$$=$1;}
			| transientformula	{$$=$1;}
			| lformula		{$$=$1;}
			| eformula		{$$=$1;}
			| cformula		{$$=$1;}
			| yformula		{$$=$1;}
			;
bracedformula		: LEFT_PARENTHESIS stateformula RIGHT_PARENTHESIS	{$$=$2;}
			;
propositionformula	: TTRUE
			{
				$$ = createTrueBitset();
			}
			| FFALSE
			{
				$$ = createFalseBitset();
			}
			| ATOMIC_PROPOSITION
			{
				$$ = getStatesSetByLabel($1);
			}
			;
steadyformula		: STEADY_STATE_F LEFT_CURLY_BRACKET comparator DOUBLE_VALUE RIGHT_CURLY_BRACKET LEFT_SQUARE_BRACKET stateformula RIGHT_SQUARE_BRACKET 
			{
				$$ = getSteadyStateOperatorResultBitset($7, $4, $3);
			}
			;
transientformula	: PROBABILITY_F LEFT_CURLY_BRACKET comparator DOUBLE_VALUE RIGHT_CURLY_BRACKET LEFT_SQUARE_BRACKET pathformula RIGHT_SQUARE_BRACKET
			{
				$$ = getProbabilityOperatorResultBitset($7, $4, $3);
			}
			;
pathformula		: LEFT_PARENTHESIS pathformula RIGHT_PARENTHESIS	{$$=$2;}
			| untilformula		{$$=$1;}
			| nextformula		{$$=$1;}
			;
untilformula		: stateformula UNTIL_F stateformula
			{
				$$ = getUnboundedUntilProbability($1, $3);
			}
			| stateformula UNTIL_F LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET stateformula 
			{
				$$ = getTimeBoundedUntilProbability($1, $8, $4, $6);
			}
			| stateformula UNTIL_F LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET stateformula 
			{
				$$ = getTimeAndRewardBoundedUntilProbability($1, $13, $4, $6, $9, $11);
			}
			;
nextformula		: NEXT_F stateformula 
			{
				$$ = getNextProbability($2);
			}
			| NEXT_F LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET stateformula 
			{
				$$ = getTimeBoundedNextProbability($7, $3, $5);
			}
			| NEXT_F LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET stateformula
			{
				$$ = getTimeAndRewardBoundedNextProbability($12, $3, $5, $8, $10);
			}
			;
lformula		: LONG_RUN_F LEFT_CURLY_BRACKET comparator DOUBLE_VALUE RIGHT_CURLY_BRACKET LEFT_SQUARE_BRACKET stateformula RIGHT_SQUARE_BRACKET 
			{
				$$ = getSteadyStateOperatorResultBitset($7, $4, $3);
			}
			;
eformula		: EXPECTED_REWARD_RATE_F LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET stateformula RIGHT_SQUARE_BRACKET 
			{
				$$ = getExpectedRewardEpochRateInterval($8, 0, $3, $5);
			}
			| EXPECTED_REWARD_RATE_F LEFT_SQUARE_BRACKET DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET stateformula RIGHT_SQUARE_BRACKET 
			{
				$$ = getExpectedRewardEpochRateInterval($11, $3, $6, $8);
			}
			;
cformula		:INSTANTANEOUS_REWARD_F LEFT_SQUARE_BRACKET DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET stateformula RIGHT_SQUARE_BRACKET 
			{
				$$ = getInstantaneousRewardEpochRateInterval($11, $3, $6, $8);
			}
			; 
yformula		: EXPECTED_ACCUMULATED_REWARD_F LEFT_SQUARE_BRACKET DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET DOUBLE_VALUE COMMA DOUBLE_VALUE RIGHT_SQUARE_BRACKET LEFT_SQUARE_BRACKET stateformula RIGHT_SQUARE_BRACKET 
			{
				$$ = getExpectedAccumulatedRewardEpochRateInterval($11, $3, $6, $8);
			}
			;
connective		: AND	{$$=AND;}
			| OR	{$$=OR;}
			| IMPLIES {$$=IMPLIES;}
			;
comparator		: GREATER	{$$=GREATER;}
			| GREATER_OR_EQUAL	{$$=GREATER_OR_EQUAL;}
			| LESS	{$$=LESS;}
			| LESS_OR_EQUAL	{$$=LESS_OR_EQUAL;}
			;
%%
yyerror(char *s)
{
	printf("%s\n",s);
	cleanResults();
}

yywrap()
{
}
